unit InfoLesen;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFnDEF FPC}
  Windows,
{$ELSE}
  LCLIntf, LCLType, LMessages,
{$ENDIF}
  SysUtils, Forms, IniFiles, Classes, Controls,
  Dialogs, StdCtrls, Graphics, ComCtrls, ExtCtrls,

  MediaInfoDll;

type
  TForm1 = class(TForm)
    OpenDialog1: TOpenDialog;
    PageMain: TPageControl;

    TabGeneral: TTabSheet;
    LabelG1: TLabel;
    LabelG2: TLabel;
    LabelG3: TLabel;
    LabelG4: TLabel;
    LabelG5: TLabel;
    EditG1: TEdit;
    EditG2: TEdit;
    EditG3: TEdit;
    EditG4: TEdit;
    EditG5: TEdit;

    TabVideo: TTabSheet;
    LabelV1: TLabel;
    EditV1: TEdit;
    LabelV4: TLabel;
    LabelV7: TLabel;
    EditV3: TEdit;
    EditV5: TEdit;
    LabelV2: TLabel;
    LabelV5: TLabel;
    EditV2: TEdit;
    EditV4: TEdit;

    TabAudio1: TTabSheet;
    LabelA11: TLabel;
    LabelA12: TLabel;
    LabelA13: TLabel;
    EditA11: TEdit;
    EditA12: TEdit;
    EditA13: TEdit;

    TabAudio2: TTabSheet;
    LabelA21: TLabel;
    LabelA22: TLabel;
    LabelA23: TLabel;
    EditA21: TEdit;
    EditA22: TEdit;
    EditA23: TEdit;

    Button1: TButton;
    DateiEdit: TEdit;

    Label1: TLabel;
    Bevel1: TBevel;
    TabAudio3: TTabSheet;
    EditA33: TEdit;
    EditA32: TEdit;
    EditA31: TEdit;
    LabelA33: TLabel;
    LabelA32: TLabel;
    LabelA31: TLabel;
    TabAudio4: TTabSheet;
    EditA43: TEdit;
    EditA42: TEdit;
    EditA41: TEdit;
    LabelA43: TLabel;
    LabelA42: TLabel;
    LabelA41: TLabel;

    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1            : TForm1;
  MediaInfoLibVer  : Byte;
  fx, fy, x, y     : Integer;
  Datei, ProgPfad  : String;

implementation

{$IFnDEF FPC}
  {$R *.dfm}
{$ELSE}
  {$R *.lfm}
{$ENDIF}

uses
  dynlibs, LazUtf8;

  //***********************************************************************//
 // Ist mindestens Windows 2000 installiert?                              //
//***********************************************************************//
function minWin2000: Boolean;
begin
  Result := true;
  {
var
  os         : TOSVersionInfo;
begin
  os.dwOSVersionInfoSize := sizeof(os);
  GetVersionEx(os);
  if (os.dwPlatformId = VER_PLATFORM_WIN32_NT) and
     (os.dwMajorVersion >= 5)
  then minWin2000 := True
  else minWin2000 := False;}
end;

  //***********************************************************************//
 // Welche MediaInfo.DLL-Version steht zur Verfügung?                     //
//***********************************************************************//
function GibMediaInfoVer: Boolean;
Const
  szNameDLL                       = 'MediaInfo.DLL';
  szNameMediaInfo_Info_Version    = 'MediaInfo_Info_Version';
  szNameMediaInfo_Option          = 'MediaInfo_Option';

Type
  TFNMediaInfo_Info_Version = function
    : PChar cdecl  {$IFDEF WIN32} stdcall {$ENDIF};
  TFNMediaInfo_Option = function
    (Handle: Cardinal; Option: PWideChar; Value: PWideChar): PWideChar cdecl
    {$IFDEF WIN32} stdcall {$ENDIF};

var
  lib                       : THandle;
  Einstieg                  : Boolean;
  VersionString             : String;
  MediaInfo_Info_Version    : TFNMediaInfo_Info_Version;
  MediaInfo_Option          : TFNMediaInfo_Option;

  function String_holen : String;
  begin
    Result := '';

    @MediaInfo_Info_Version  := NIL;
    @MediaInfo_Option        := NIL;
    lib := LoadLibrary(szNameDLL);
    if lib <> 0
    then try
      @MediaInfo_Info_Version := GetProcAddress(lib, szNameMediaInfo_Info_Version);
      if Assigned(MediaInfo_Info_Version)
      then Result := MediaInfo_Info_Version();

      if pos('MediaInfoLib', Result) > 0 then Exit;

      @MediaInfo_Option := GetProcAddress(lib, szNameMediaInfo_Option);
      if Assigned(MediaInfo_Option)
      then Result := MediaInfo_Option (0, 'Info_Version', '');
    except
    end;
  end;

begin
  Result          := False;
  VersionString   := String_holen;

  if VersionString = ''
  then
    VersionString := 'Keine MediaInfo.Dll gefunden.'
  else begin
    if (Pos('0.7', VersionString) > 0)
    then
      if minWin2000
      then Result := True;
  end;

  if result
  then
    Form1.Label1.Font.Color := clBlack
  else
    Form1.Label1.Font.Color := clRed;

  Form1.Label1.Caption := VersionString;
end;

function Spieldauer : String;
var
  Dateiinhalt : TStringList;
  Dateiname   : String;
  ZweitHandle : Cardinal;
begin
  Dateiinhalt := TStringList.Create;
  Dateiinhalt.Add('DirectShowSource("' + Datei + '")');
  try
    Dateiname := ProgPfad + '\video.avs';
    Dateiinhalt.SaveToFile (Dateiname);
    If FileExists (Dateiname)
    then begin
      ZweitHandle  := MediaInfo_New;
      MediaInfoA_Open(ZweitHandle, PChar(Dateiname));
      Spieldauer := MediaInfoA_Get(ZweitHandle, 0, 0, 'PlayTime/String1', 1, 0);
      MediaInfoA_Close(ZweitHandle);
    end
    else Spieldauer := '';
  except
    If FileExists (Dateiname) Then DeleteFile (Dateiname);
  end;
  Dateiinhalt.Free;
end;

  //***********************************************************************//
 // Form1 erstellen                                                       //
//***********************************************************************//
procedure TForm1.FormCreate(Sender: TObject);
var
  Ini      : TIniFile;
  IniDatei : String;
  x, y     : Integer;
begin
  ProgPfad      := ExtractFileDir(ParamStr(0));
  Form1.Caption := 'GUI für MediaInfoLib v0.7.*.*';

  { INI-Datei laden }
  IniDatei := IncludeTrailingBackslash(ProgPfad + 'GUIMediaInfoLib.ini');

  ini:=TIniFile.Create(IniDatei);

  { Fensterposition lesen, wenn nicht angegeben Positon 100, 100 }
  x := ini.ReadInteger('Position','Links',100);
  y := ini.ReadInteger('Position','Oben',100);

  { Zuletzt geöffnetes Verzeichnis lesen }
  OpenDialog1.InitialDir := ini.ReadString('Pfad','Video','');

  ini.Free;

  { Position des Fensters setzen }
  Form1.Left       := x;
  Form1.Top        := y;
  Button1.Enabled  := GibMediaInfoVer;
  LabelG1.Caption  := 'Dateigröße';
  LabelG2.Caption  := 'Bitrate';
  LabelG3.Caption  := 'Spieldauer';
  LabelG4.Caption  := 'Tonspuren';
  LabelG5.Caption  := 'Format';
  LabelV1.Caption  := 'Codec';
  LabelV2.Caption  := 'Bildgröße';
  LabelV4.Caption  := 'Bitrate';
  LabelV5.Caption  := 'AspectRatio';
  LabelV7.Caption  := 'Bilder/s';
  LabelA11.Caption := 'Codec';
  LabelA12.Caption := 'Bitrate';
  LabelA13.Caption := 'Eigensch.';
  LabelA21.Caption := 'Codec';
  LabelA22.Caption := 'Bitrate';
  LabelA23.Caption := 'Eigensch.';
  LabelA31.Caption := 'Codec';
  LabelA32.Caption := 'Bitrate';
  LabelA33.Caption := 'Eigensch.';
  LabelA41.Caption := 'Codec';
  LabelA42.Caption := 'Bitrate';
  LabelA43.Caption := 'Eigensch.';

  PageMain.ActivePage  := TabGeneral;
  TabVideo.TabVisible  := False;
  TabAudio1.TabVisible := False;
  TabAudio2.TabVisible := False;
  TabAudio3.TabVisible := False;
  TabAudio4.TabVisible := False;

end;

  //***********************************************************************//
 // Datei laden und Informationen holen.                                  //
//***********************************************************************//
procedure TForm1.Button1Click(Sender: TObject);
var
  Handle           : Cardinal;
  i,
  Videospuren,
  Audiospuren      : Integer;
  Hilf             : array[1..4] of Char;
  RiffString,
  Hilfstring       : String;
  Video            : File;

begin

  { Dateidialog zum Video laden öffnen }
  if OpenDialog1.Execute
  then begin
    { Dateiname lesen und in Formular eintragen }
    Datei          := OpenDialog1.Filename;
    DateiEdit.Text := ExtractFileName(Datei);
  end;

  { Existiert die Datei wird versucht, Informationen auszulesen. Können
    die Informationen nicht gelesen werden, werden Leerstrings zurück-
    gegeben. }
  if FileExists(Datei)
  then begin

    TabVideo.TabVisible  := False;
    TabAudio1.TabVisible := False;
    TabAudio2.TabVisible := False;
    TabAudio3.TabVisible := False;
    TabAudio4.TabVisible := False;

    if GetEntryPoints
    then try

      { Neue Instanz anlegen }

      Handle := MediaInfoA_New;

      { Datei öffnen }

      MediaInfoA_Open(Handle, PChar(Datei));

      { Generalinformationen: }
      { ********************* }

      { Format }
      Hilfstring  := MediaInfoA_Get(Handle, 0, 0, 'Format', 1, 0);
      if Hilfstring <> ''
      then begin
        if Hilfstring = 'RIFF'
        then begin
          try
            FileMode := fmOpenRead;
            AssignFile (Video, Datei);
            Reset (Video, 1);
          except
            EXIT;
          end;
          Seek (Video, 8);
          BlockRead (Video, Hilf, 4);
          RiffString := '';
          for i := 1 to 4 do
            RiffString := RiffString + Hilf[i];
          Hilfstring := Hilfstring + '-' + RiffString;
        end;
        EditG5.Text := Hilfstring
                     + ': '
                     + MediaInfoA_Get(Handle, 0, 0, 'Format/String', 1, 0);
      end
      else
        EditG5.Text := MediaInfoA_Get(Handle, 0, 0, 'Format/String', 1, 0);

      { Dateigröße }
      EditG1.Text := MediaInfoA_Get(Handle, 0, 0, 'FileSize/String4', 1, 0);

      { Gesamtbitrate }
      EditG2.Text := MediaInfoA_Get(Handle, 0, 0, 'Bitrate', 1, 0);

      { Spieldauer }
      EditG3.Text := MediaInfoA_Get(Handle, 0, 0, 'PlayTime/String', 1, 0);

      HilfString := Copy (Hilfstring, 0, 4);
      if (EditG3.Text = '') and (Hilfstring = 'MPEG')
      then EditG3.Text := Spieldauer;

      { Anzahl Tonspuren }
      EditG4.Text := MediaInfo_Get(Handle, 0, 0, 'AudioCount', 1, 0);
      Audiospuren := StrToInt(EditG4.Text);
      Videospuren := StrToInt(MediaInfo_Get(Handle, 0, 0, 'VideoCount', 1, 0));

      { Videoinformationen: }
      { ******************* }

      If Videospuren > 0
      then begin

        TabVideo.TabVisible := True;

        { Videocodec }
        Hilfstring  := MediaInfoA_Get(Handle, 1, 0, 'Codec', 1, 0);
        if Hilfstring <> ''
        then
          EditV1.Text := Hilfstring
                       + ': '
                       + MediaInfoA_Get(Handle, 1, 0, 'Codec/String', 1, 0)
        else
          EditV1.Text := MediaInfoA_Get(Handle, 1, 0, 'Codec/String', 1, 0);

        if (EditV1.Text = '') and (RiffString = 'VDRM')
        then EditV1.Text := 'VirtualDub Frameserver';

        { Bildauflösung }
        EditV2.Text := MediaInfoA_Get(Handle, 1, 0, 'Width', 1, 0)
                     + ' x '
                     + MediaInfoA_Get(Handle, 1, 0, 'Height', 1, 0);

        { Bitrate }
        EditV3.Text := MediaInfoA_Get(Handle, 1, 0, 'Bitrate', 1, 0);
        if EditV3.Text = ''
        then EditV3.Text := MediaInfoA_Get(Handle, 1, 0, 'BitRate/String', 1, 0);
        Hilfstring := MediaInfoA_Get(Handle, 1, 0, 'BitRate/Mode', 1, 0);

        if Hilfstring <> ''
        then EditV3.Text := EditV3.Text + ' (' + Hilfstring + ')';

        Hilfstring := EditV3.Text;
        if pos('-', Hilfstring) > 0
        then EditV3.Text := '';

        { Aspect Ratio }
        EditV4.Text := MediaInfoA_Get(Handle, 1, 0, 'AspectRatio', 1, 0);
        Hilfstring := MediaInfoA_Get(Handle, 1, 0, 'AspectRatio/String', 1, 0);
        if Hilfstring <> ''
        then EditV4.Text := EditV4.Text + ' (' + Hilfstring + ')';

        { Bildwiederholrate }
        EditV5.Text := MediaInfoA_Get(Handle, 1, 0, 'FrameRate', 1, 0);
      end;

      { Audioinformationen Stream 1: }
      { **************************** }

      If Audiospuren > 0
      then begin

        TabAudio1.TabVisible := True;

        {Codec }
        Hilfstring := MediaInfoA_Get(Handle, 2, 0, 'Codec', 1, 0);

        If Hilfstring <> ''
        then EditA11.Text := Hilfstring
        else EditA11.Text := '?';

        EditA11.Text := EditA11.Text
                      + ': '
                      + MediaInfoA_Get(Handle, 2, 0, 'Codec/String', 1, 0);

        { Bitrate }
        EditA12.Text := MediaInfoA_Get(Handle, 2, 0, 'Bitrate', 1, 0);
        if EditA12.Text = ''
        then EditA12.Text := MediaInfoA_Get(Handle, 2, 0, 'BitRate/String', 1, 0);

        { Samplerate, Kanäle und Mode }
        EditA13.Text := MediaInfoA_Get(Handle, 2, 0, 'SamplingRate/String', 1, 0)
                      + ' / '
                      + MediaInfoA_Get(Handle, 2, 0, 'Channels', 1, 0)
                      + 'ch '
                      + MediaInfoA_Get(Handle, 2, 0, 'ChannelMode', 1, 0);
      end;

      { Audioinformationen Stream 2: }
      { **************************** }

      If Audiospuren > 1
      then begin

        TabAudio2.TabVisible := true;

        { Codec }
        Hilfstring := MediaInfoA_Get(Handle, 2, 1, 'Codec', 1, 0);

        If Hilfstring <> ''
        then EditA21.Text := Hilfstring
        else EditA21.Text := '?';

        EditA21.Text := EditA21.Text
                      + ': '
                      + MediaInfoA_Get(Handle, 2, 1, 'Codec/String', 1, 0);

        { Bitrate }
        EditA22.Text := MediaInfoA_Get(Handle, 2, 1, 'Bitrate', 1, 0);
        if EditA22.Text = ''
        then EditA22.Text := MediaInfoA_Get(Handle, 2, 1, 'BitRate/String', 1, 0);

        { Samplerate, Kanäle und mode }
        EditA23.Text := MediaInfoA_Get(Handle, 2, 1, 'SamplingRate/String', 1, 0)
                      + ' / '
                      + MediaInfoA_Get(Handle, 2, 1, 'Channels', 1, 0)
                      + 'ch '
                      + MediaInfoA_Get(Handle, 2, 1, 'ChannelMode', 1, 0);
      end;

      { Audioinformationen Stream 3: }
      { **************************** }

      If Audiospuren > 2
      then begin

        TabAudio3.TabVisible := True;

        {Codec }
        Hilfstring := MediaInfoA_Get(Handle, 2, 2, 'Codec', 1, 0);

        If Hilfstring <> ''
        then EditA31.Text := Hilfstring
        else EditA31.Text := '?';

        EditA31.Text := EditA31.Text
                      + ': '
                      + MediaInfoA_Get(Handle, 2, 2, 'Codec/String', 1, 0);

        { Bitrate }
        EditA32.Text := MediaInfoA_Get(Handle, 2, 2, 'Bitrate', 1, 0);
        if EditA32.Text = ''
        then EditA32.Text := MediaInfoA_Get(Handle, 2, 2, 'BitRate/String', 1, 0);

        { Samplerate, Kanäle und Mode }
        EditA33.Text := MediaInfoA_Get(Handle, 2, 2, 'SamplingRate/String', 1, 0)
                      + ' / '
                      + MediaInfoA_Get(Handle, 2, 2, 'Channels', 1, 0)
                      + 'ch '
                      + MediaInfoA_Get(Handle, 2, 2, 'ChannelMode', 1, 0);
      end;

      { Audioinformationen Stream 4: }
      { **************************** }

      If Audiospuren > 3
      then begin

        TabAudio4.TabVisible := True;

        {Codec }
        Hilfstring := MediaInfoA_Get(Handle, 2, 3, 'Codec', 1, 0);

        If Hilfstring <> ''
        then EditA41.Text := Hilfstring
        else EditA41.Text := '?';

        EditA41.Text := EditA41.Text
                      + ': '
                      + MediaInfoA_Get(Handle, 2, 3, 'Codec/String', 1, 0);

        { Bitrate }
        EditA42.Text := MediaInfoA_Get(Handle, 2, 3, 'Bitrate', 1, 0);
        if EditA42.Text = ''
        then EditA42.Text := MediaInfoA_Get(Handle, 2, 3, 'BitRate/String', 1, 0);

        { Samplerate, Kanäle und Mode }
        EditA43.Text := MediaInfoA_Get(Handle, 2, 3, 'SamplingRate/String', 1, 0)
                      + ' / '
                      + MediaInfoA_Get(Handle, 2, 3, 'Channels', 1, 0)
                      + 'ch '
                      + MediaInfoA_Get(Handle, 2, 3, 'ChannelMode', 1, 0);
      end;

      { Mediadatei schließen }

      MediaInfoA_Close(Handle);

      { Instant löschen }

      MediaInfoA_Delete(Handle);

    except
      ShowMessage ('Beim Analysieren der Datei ' + ExtractFileName(Datei) +
                   ' ist ein Fehler aufgetreten.');
    end;

  end;
end;

  //***********************************************************************//
 // Form schließen.                                                       //
//***********************************************************************//
procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Ini      : TIniFile;
  IniDatei,
  Hilf     : String;
  x, y     : Integer;
begin
  x := Form1.Left;
  y := Form1.Top;
  if Length(ProgPfad) < 4
  then IniDatei := ProgPfad + 'GUIMediaInfoLib.ini'
  else IniDatei := ProgPfad + '\GUIMediaInfoLib.ini';
  try
    ini:=TIniFile.Create(IniDatei);
    ini.WriteInteger('Position','Links',x);
    ini.WriteInteger('Position','Oben',y);
    if Hilf <> '' then
    ini.WriteString('Pfad','Video',Hilf);
    ini.Free;
  except
  end;
end;

end.
